package net.vtstar.zhongtong.avi.equipment.job;

import lombok.extern.slf4j.Slf4j;
import net.vtstar.scada.base.equipmgt.domain.Enum.EquipStatusEnum;
import net.vtstar.scada.base.equipmgt.service.EquipDataService;
import net.vtstar.scada.base.gather.domain.EquipmentData;
import net.vtstar.scada.base.redis.service.SimpleRedisService;
import net.vtstar.scada.base.redis.util.EquipDataUtil;
import net.vtstar.user.mybatis.provider.Operation;
import net.vtstar.user.mybatis.provider.WhereFilter;
import net.vtstar.zhongtong.avi.equipment.domain.EquipInfo;
import net.vtstar.zhongtong.avi.equipment.domain.RuntimeRecord;
import net.vtstar.zhongtong.avi.equipment.mapper.BasicInfoMapper;
import net.vtstar.zhongtong.avi.equipment.mapper.RuntimeRecordMapper;
import net.vtstar.zhongtong.avi.equipment.service.RunRecordService;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

@Slf4j
@Component
@DisallowConcurrentExecution
public class CleanAndStatisticsJob extends QuartzJobBean {


    @Resource(name = "redisTemplate")
    private RedisTemplate redisTemplate;
    @Autowired
    private EquipDataService equipDataService;
    @Autowired
    private SimpleRedisService simpleRedisService;
    @Autowired
    private RunRecordService runRecordService;
    @Autowired
    private RuntimeRecordMapper runtimeRecordMapper;
    @Autowired
    private BasicInfoMapper basicInfoMapper;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {

        log.info("_______________________________________________________________");
        log.info("开始清洗设备状态数据...");
        Set<String> keys = redisTemplate.keys(EquipDataUtil.getStatusKey("*"));
        if (CollectionUtils.isEmpty(keys)) {
            return;
        }
        log.info("开始清洗设备状态数据 key.size: {}", keys.size());

        for (String key : keys) {
            List<EquipmentData> dataList = (List<EquipmentData>) simpleRedisService.getListDelDefaultSize(key);
            if (CollectionUtils.isEmpty(dataList)) {
                continue;
            }

            equipDataService.updateLastStatus(dataList.get(dataList.size() - 1));

            String equipCode = dataList.get(0).getEquipCode();
            EquipmentData last = equipDataService.getLast(equipCode);
            final EquipmentData first = last;
            int index = 1;
            long totalRuntime = 0;
            for (int i = 0; i < dataList.size(); i++) {

                if (last == null) {
                    last = dataList.get(0);
                    continue;
                }

                if (last.getBeginTime() == null || last.getEquipStatus() == null) {
                    redisTemplate.delete(key);
                    break;
                }

                EquipmentData d = dataList.get(i);
                last.setEndTime(d.getBeginTime());

                if (d.getEquipStatus() != last.getEquipStatus()) {

                    if (index == 1) {
                        equipDataService.update(last);
                    } else {
                        // 保存
                        equipDataService.save(last);
                    }

                    //计算累计运行时间
                    if (last.getEquipStatus().equals(EquipStatusEnum.RUN)) {
                        long runtime = last.getEndTime().getTime() - last.getBeginTime().getTime();
                        totalRuntime += runtime;
                    }
                    last = d;
                    index++;
                }
            }

            if (first != null && first.getBeginTime().equals(last.getBeginTime())) {
                equipDataService.update(last);
            } else {
                equipDataService.save(last);
            }

            RuntimeRecord lastRecord = runRecordService.getLastByCode(equipCode);
            RuntimeRecord runtimeRecord = new RuntimeRecord();
            runtimeRecord.setCreateTime(new Date());
            BigDecimal hours = new BigDecimal(totalRuntime).divide(new BigDecimal(3600000), 4, BigDecimal.ROUND_HALF_UP);
            runtimeRecord.setCurrentRuntime(hours);
            BigDecimal total = BigDecimal.ZERO;
            if (null != lastRecord) {
                total = lastRecord.getTotalRuntime();
            }
            runtimeRecord.setTotalRuntime(total.add(hours));
            List<EquipInfo> list = basicInfoMapper.findList(new WhereFilter() {{
                addFilter(EquipInfo.class, "equip_code", Operation.EQUAL, equipCode);
            }}, EquipInfo.class);
            EquipInfo eqp = list.get(0);
            runtimeRecord.setMachineId(eqp.getId());
            runtimeRecord.setWorkshopCode(eqp.getWorkshopCode());
            List<RuntimeRecord> records = new ArrayList<>();
            records.add(runtimeRecord);
            runtimeRecordMapper.insertLists(records);
        }
    }
}
